# RTLS Solution Services

## Common

### Description

This GitHub repository is a comprehensive solution for building a back-end services that includes
a `Solutions Positions Manager` and `Solutions Subscriptions Manager`.
The repository is organized into three main components:

- `rtls-solutions-positions-manager`: Service that handles ZMQ messages from channel `$pos` and saves positions to the
  database.
- `rtls-solutions-subscriptions-manager`: Implements WebSocket API and re translate ZMQ messages from channels
  `$pos`, `#notify`, `#alarm`, `#param_p`, `#zone_event`, `#sensor`, `#ota` to WebSocket channels.

### REST API Documentation

[Documentation](https://github.com/rpplabs/rtls-frontend/blob/dev/docs/api/websocket.md)

### Technologies

- `rtls-solutions-positions-manager`: NodeJs, Express, MariaDB, ZMQ
- `rtls-solutions-subscriptions-manager`: Nodejs, WebSocket, ZMQ

### Requirements

- Ubuntu 22.04
- NodeJs v20
- MariaDB
- Redis

### Project local setup

- clone git repository
- setup projects `rtls-frontend` and `rtls-historical-api-server`
- install `NodeJs` via `nvm`

```shell
curl -o- https://raw.githubusercontent.com/nvm-sh/nvm/v0.39.3/install.sh | bash
nvm install 20.5.1

```

- or install `NodeJs`

```shell
curl -fsSL https://deb.nodesource.com/setup_20.x | sudo bash -
apt-get install -y nodejs

```

- install `MariaDB`

```shell
sudo apt update
sudo apt install mariadb-server

```

- install `Redis`

```shell
sudo apt update
sudo apt install redis-server

```

- install positions manager node packages `cd ./apps/positions-manager` and  `npm install`
- install positions manager node packages `cd ./apps/subscriptions-manager` and  `npm install`
- run positions manager `npm run pm:start`
- run subscriptions manager `npm run sm:start`

### Configure

For packages configuration create `.env` files:

```shell
cp apps/positions-manager/.env.defaults apps/positions-manager/.env
cp apps/subscriptions-manager/.env.defaults apps/subscriptions-manager/.env

```

and update environment variables (e.g. database password)

### Version increment

Scripts for version increment

- subscriptions-manager

```shell
yarn version:patch:sm

```

- positions-manager

```shell
yarn version:patch:pm

```

### Build packages

For build package require to push branch

- positions-manager `master_pm`
- subscriptions-manager `master_sm`

## Conventions

### Code style

#### Formatting rules

- please, follow the eslint rules described in project (with this purpose setting up your IDE with
  the [.eslintrc](.eslintrc));

#### Naming rules

- please name the variables and constants in a human-readable manner (ex. use `getUsersSql` instead of `sql1`);
- please use the `camelCase` style in name the variables and constants (ex. `getUsersSql` instead of `get_users_sql`);
- please use upper case naming for constants that are not changeable in project (ex. `MAX_ROWS_NUMBER`);
- please use `JavaScript` conventions in naming the classes, constructors(functions) and others entities (classes should
  be named wih the first capital latter);

### Tests

#### Unit tests

- unit tests should be places at the same level as the tested file
- unit test name should include the name of the file that is contains covered functionality and obey a
  pattern `{filename}.test.mjs`
- run tests with the command `yarn test:unit` in application
  directory (`apps/subscriptions-manager`, `apps/positions-manager`)

#### Integration tests

- e2e tests should be places at the `e2e/integration` folder of each application
- e2e test name should include the name of the entity that is testing and obey a pattern `{filename}.test.mjs`
- before running e2e test you should run application in test mode: `yarn Server:test` in application
  directory (`apps/subscriptions-manager`, `apps/positions-manager`)
- run tests with the command `yarn test:integration` in application
  directory (`apps/subscriptions-manager`, `apps/positions-manager`)
